/* compile with
   gcc -o recentfiles recentfiles.c `pkg-config --cflags --libs gtk+-2.0 libglade-2.0 libnotify`
*/
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libnotify/notify.h>

GtkWidget *filechooserbutton;
GtkWidget *menu;
GtkWidget *checkbutton;
GtkStatusIcon *icon;
GtkActionGroup *actions_recent;
GtkUIManager *ui_manager;
gint recent_menu_id;

void
on_button_add_clicked (GtkButton *button,
		       gpointer user_data)
{
	GtkRecentManager *manager;
	NotifyNotification *notification;
	gchar *msg = NULL;
	gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooserbutton));

	if (filename) {
		manager = GTK_RECENT_MANAGER (user_data);
		gtk_recent_manager_add_item (manager, filename);
		msg = g_strdup_printf ("Se ha agregado el archivo %s a la lista de archivos recientes.",
				       filename);

		g_free (filename);
	}

	if (msg) {
		notification = notify_notification_new ("Nuevo archivo reciente",
							msg,
							NULL,
							NULL);
		g_free (msg);
	
		notify_notification_set_timeout (notification, 5000);

		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton))) {
			int x, y;
			GdkRectangle rectangle;

			gtk_status_icon_get_geometry (icon, NULL, &rectangle, NULL);
			
			x = rectangle.x + rectangle.width/2;
			y = rectangle.y + rectangle.height/2;
			
			notify_notification_set_hint_int32 (notification, "x", x);
			notify_notification_set_hint_int32 (notification, "y", y);
		}
		
		
		if (!notify_notification_show (notification, NULL)) {
			fprintf(stderr, "No se pudo mostrar la notificación\n");
		}

		g_object_unref (notification);
	}
}

void
on_checkbutton_toggled (GtkToggleButton *toggle,
			gpointer userdata)
{
	GtkStatusIcon *icon = GTK_STATUS_ICON (userdata);

	gtk_status_icon_set_visible (icon, gtk_toggle_button_get_active (toggle));
}

static gint
sort_recents_mru (GtkRecentInfo *a, GtkRecentInfo *b)
{
	return (gtk_recent_info_get_modified (a) < gtk_recent_info_get_modified (b));
}

void
on_manager_changed (GtkRecentManager *manager,
		    gpointer user_data)
{
	GList *items = NULL;
	GList *li;
	gint count_recent = 0;
	
	items = gtk_recent_manager_get_items (manager);
	items = g_list_sort (items, (GCompareFunc) sort_recents_mru);

	for (li = items; li != NULL && count_recent < 10; li = li->next, count_recent ++) {
		gchar *action_name;
		gchar *label;
		gchar *tip;
		gchar **display_name;
		gchar *label_filename;
		GtkAction *action;
		GtkRecentInfo *info = li->data;
		
		action_name = g_strdup_printf ("recent-info-%d", count_recent);
		display_name = g_strsplit (gtk_recent_info_get_display_name (info), "_", -1);
		label_filename = g_strjoinv ("__", display_name);
		label = g_strdup_printf ("_%d. %s", count_recent + 1, label_filename);
		g_free (label_filename);
		g_strfreev (display_name);

		tip = gtk_recent_info_get_uri_display (info);
		
		action = gtk_action_new (action_name, label, tip, NULL);
		
		g_object_set_data_full (G_OBJECT (action), "gtk-recent-info",
					gtk_recent_info_ref (info),
					(GDestroyNotify) gtk_recent_info_unref);
		
		g_object_set (G_OBJECT (action), "icon-name", "gnome-mime-image", NULL);
				
		gtk_action_group_add_action (actions_recent, action);
		
		g_object_unref (action);

		gtk_ui_manager_add_ui (ui_manager, recent_menu_id,
				       "/MainMenu/File/RecentDocuments",
				       action_name, action_name,
				       GTK_UI_MANAGER_AUTO, FALSE);

		g_free (action_name);
		g_free (label);
		g_free (tip);
	}

	g_list_foreach (items, (GFunc) gtk_recent_info_unref, NULL);
	g_list_free (items);

}

static const GtkActionEntry action_entries[] = {
	{ "File",  NULL, "Archivo" },
	{ "FileOpen", GTK_STOCK_OPEN,  "Abrir", "<control>O", 
	  "Open a file"},
	{ "FileClose", GTK_STOCK_CLOSE, "Cerrar", "<control>W",  
	  "Close window"},
	{ "FileSave", GTK_STOCK_SAVE, "Guardar", "<control>s", 
	  NULL},
	{ "FileSaveAs", GTK_STOCK_SAVE_AS, "Save _As...", "<control><shift>s", 
	  NULL},
	{ "FilePageSetup", "stock_print-setup", "Page Set_up...", NULL, 
	  NULL},
	{ "FilePrint", GTK_STOCK_PRINT, "Print...", "<control>p", 
	  NULL},
};

int
main (gint argc, gchar **argv)
{
	GladeXML *xml;
	GtkWidget *window;
	GtkWidget *button;
	GtkWidget *menubar;
	GtkWidget *vbox;
	
	GtkActionGroup *actions;
	
	GtkRecentManager *manager;

	/* iniciar el sistema de notificaciones */
	notify_init ("GuadecHispana");

	/* iniciar gtk+ */
	gtk_init (&argc, &argv);

	/* obtener los elementos de la interfaz desde el .glade */
	xml = glade_xml_new ("recentfilesui.glade",
			     NULL, NULL);

	window = glade_xml_get_widget (xml, "window1");
	button = glade_xml_get_widget (xml, "button1");
	vbox = glade_xml_get_widget (xml, "vbox1");
	checkbutton = glade_xml_get_widget (xml, "checkbutton1");
	filechooserbutton = glade_xml_get_widget (xml, "filechooserbutton1");
	menu = glade_xml_get_widget (xml, "menu1");

	/* crear el menu desde un archivo .xml */
	ui_manager = gtk_ui_manager_new ();
	if (!gtk_ui_manager_add_ui_from_file (ui_manager, "ui.xml", NULL)) {
                g_warning ("building menus failed");
        }

	/* nos preocupamos del sistema de archivos recientes */
	manager = gtk_recent_manager_get_default ();
	/* creamos las acciones para el menu */
	actions = gtk_action_group_new ("File");
	gtk_action_group_add_actions (actions,
				      action_entries,
				      G_N_ELEMENTS (action_entries), 
				      NULL);
	gtk_ui_manager_insert_action_group (ui_manager, actions, 0);
	
	/* creamos las acciones para las acciones de archivos recientes */
	actions_recent = gtk_action_group_new ("RecentFilesActions");
	/* obtenemos un id para luego poder mezclar las nuevas acciones */
	recent_menu_id = gtk_ui_manager_new_merge_id (ui_manager);
	gtk_ui_manager_insert_action_group (ui_manager, actions_recent, 0);

	/* menu listo, lo obtenemos y anadimos a la interfaz */
	menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
	gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
	
	g_signal_connect (button, "clicked",
			  G_CALLBACK (on_button_add_clicked),
			  manager);

	g_signal_connect (manager, "changed",
			  G_CALLBACK (on_manager_changed),
			  actions_recent);

	icon = gtk_status_icon_new_from_stock (GTK_STOCK_DIALOG_WARNING);
	gtk_status_icon_set_tooltip  (icon, "Esta es una advertencia...");
	gtk_status_icon_set_visible (icon, TRUE);

	g_signal_connect (checkbutton, "toggled",
			  G_CALLBACK (on_checkbutton_toggled),
			  icon);

	gtk_widget_show (window);

	gtk_main ();

	return 0;
}
